@@ -249,6 +249,10 @@ sub std_include {
 sub localize_vars {
     my($parser, $symbol) = @_;
 
+# should make 'WITH' optional?
+#    my $t = $parser->token;
+#    if(uc($t->id) eq "WITH" or $t->arity eq "variable") {
+#        $parser->advance() if uc($t->id) eq "WITH";
     if(uc($parser->token->id) eq "WITH") {
         $parser->advance();
         $parser->new_scope();
@@ -651,6 +655,8 @@ Unlike Template-Toolkit, calling macros requires parens (C<()>).
 
 A few methods are supported in the Xslate core.
 
+    %% any.defined()
+
     %% a.size();
     %% a.join(", ");
     %% a.reverse();
